#include <stdio.h>
#include <stdlib.h>
#include <phidget22.h>
#include "PhidgetHelperFunctions.h"


/**
* Displays info about the attached phidget channel.
* Fired when a Phidget channel with onAttachHandler registered attaches
*
* @param ph The Phidget channel that fired the attach event
* @param *ctx Context pointer
*/
static void CCONV onAttachHandler(PhidgetHandle ph, void *ctx) {
	PhidgetReturnCode prc;

	//If you are unsure how to use more than one Phidget channel with this event, we recommend going to
	//www.phidgets.com/docs/Using_Multiple_Phidgets for information

	printf("\nAttach Event: ");
	PrintChannelInformaiton(ph);

	/*
	*	Set a TargetPosition inside of the attach handler to initialize the servo's starting position.
	*	TargetPosition defines position the RC Servo will move to.
	*	TargetPosition can be set to any value from MinPosition to MaxPosition.
	*/
	prc = PhidgetRCServo_setTargetPosition((PhidgetRCServoHandle)ph, 90);
	CheckError(prc, "Setting Target Position", (PhidgetHandle *)&ph);

	/*
	*	Engage the RCServo inside of the attach handler to allow the servo to move to its target position
	*	The servo will only track a target position if it is engaged.
	*	Engaged can be set to TRUE to enable the servo, or FALSE to disable it.
	*/
	prc = PhidgetRCServo_setEngaged((PhidgetRCServoHandle)ph, 1);
	CheckError(prc, "Setting Engaged", (PhidgetHandle *)&ph);
}

/**
* Displays info about the detached phidget channel.
* Fired when a Phidget channel with onDetachHandler registered detaches
*
* @param ph The Phidget channel that fired the detach event
* @param *ctx Context pointer
*/
static void CCONV onDetachHandler(PhidgetHandle ph, void *ctx) {
	printf("\nDetach Event: ");
	PrintChannelInformaiton(ph);
}

/**
* Writes phidget error info to stderr.
* Fired when a Phidget channel with onErrorHandler registered encounters an error in the library
*
* @param ph The Phidget channel that fired the error event
* @param *ctx Context pointer
* @param errorCode the code associated with the error of enum type Phidget_ErrorEventCode
* @param *errorString string containing the description of the error fired
*/
static void CCONV onErrorHandler(PhidgetHandle phid, void *ctx, Phidget_ErrorEventCode errorCode, const char *errorString) {

	fprintf(stderr, "[Phidget Error Event] -> %s (%d)\n", errorString, errorCode);
}

/**
* Outputs the RCServo's most recently detected state change.
* Fired when a RCServo channel with onTargetPositionReachedHandler registered detects a state change
*
* @param pdih The RCServo channel that fired the TargetPositionReached event
* @param *ctx Context pointer
* @param state The reported state from the RCServo channel
*/
static void CCONV onTargetPositionReachedHandler(PhidgetRCServoHandle pdih, void *ctx, double position) {

	//If you are unsure how to use more than one Phidget channel with this event, we recommend going to
	//www.phidgets.com/docs/Using_Multiple_Phidgets for information

	printf("[Target Pos Reached Event] -> Position: %lf\n", position);
}


/**
* Prints descriptions of the available events for this class
*/
void PrintEventDescriptions() {

	printf("\n--------------------\n"
		"\n  | Target Position Reached events will call their associated function every time the RCServo controller has reached its target position.\n"
		"  | Press ENTER once you have read this message.\n");
	getchar();

	printf("\n--------------------\n");
}

/**
* Creates, configures, and opens a RCServo channel.
* Displays state change events for 10 seconds
* Closes out RCServo channel
*
* @return 0 if the program exits successfully, 1 if it exits with errors.
*/
int main() {
	PhidgetRCServoHandle ch = NULL;
	ChannelInfo channelInfo;
	PhidgetReturnCode prc;

	/*
	* Allocate a new Phidget Channel object
	*/

	prc = PhidgetRCServo_create(&ch);
	CheckError(prc, "Creating Channel", (PhidgetHandle *)&ch);

	/*
	* Set matching parameters to specify which channel to open
	*/

	//You can safely remove this line and hard-code the parameters to make the program can start without user input
	AskForDeviceParameters(&channelInfo, (PhidgetHandle *)&ch);

	prc = Phidget_setDeviceSerialNumber((PhidgetHandle)ch, channelInfo.deviceSerialNumber);
	CheckError(prc, "Setting DeviceSerialNumber", (PhidgetHandle *)&ch);

	prc = Phidget_setHubPort((PhidgetHandle)ch, channelInfo.hubPort);
	CheckError(prc, "Setting HubPort", (PhidgetHandle *)&ch);

	prc = Phidget_setChannel((PhidgetHandle)ch, channelInfo.channel);
	CheckError(prc, "Setting Channel", (PhidgetHandle *)&ch);

	if (channelInfo.netInfo.isRemote) {
		prc = Phidget_setIsRemote((PhidgetHandle)ch, channelInfo.netInfo.isRemote);
		CheckError(prc, "Setting IsRemote", (PhidgetHandle *)&ch);
		if (channelInfo.netInfo.serverDiscovery) {
			prc = PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
			CheckEnableServerDiscoveryError(prc, (PhidgetHandle *)&ch);
		} else {
			prc = PhidgetNet_addServer("Server", channelInfo.netInfo.hostname,
				channelInfo.netInfo.port, channelInfo.netInfo.password, 0);
			CheckError(prc, "Adding Server", (PhidgetHandle *)&ch);
		}
	}

	/*
	* Add event handlers before calling open so that no events are missed.
	*/

	printf("\n--------------------------------------\n");
	printf("\nSetting OnAttachHandler...\n");
	prc = Phidget_setOnAttachHandler((PhidgetHandle)ch, onAttachHandler, NULL);
	CheckError(prc, "Setting OnAttachHandler", (PhidgetHandle *)&ch);

	printf("Setting OnDetachHandler...\n");
	prc = Phidget_setOnDetachHandler((PhidgetHandle)ch, onDetachHandler, NULL);
	CheckError(prc, "Setting OnDetachHandler", (PhidgetHandle *)&ch);

	printf("Setting OnErrorHandler...\n");
	prc = Phidget_setOnErrorHandler((PhidgetHandle)ch, onErrorHandler, NULL);
	CheckError(prc, "Setting OnErrorHandler", (PhidgetHandle *)&ch);

	//This call may be harmlessly removed
	PrintEventDescriptions();

	printf("Setting OnTargetPositionReachedHandler...\n");
	prc = PhidgetRCServo_setOnTargetPositionReachedHandler(ch, onTargetPositionReachedHandler, NULL);
	CheckError(prc, "Setting OnTargetPositionReachedHandler", (PhidgetHandle *)&ch);

	/*
	* Open the channel with a timeout
	*/

	prc = Phidget_openWaitForAttachment((PhidgetHandle)ch, 5000);
	CheckOpenError(prc, (PhidgetHandle *)&ch);

	char buf[100];
	char* endptr;
	int end = 0;
	double targetPosition = 0.0;

	printf("--------------------\n"
		"\n  | RC Servo position can be controlled by setting its Target Position.\n"
		"  | By default, the target  can be a number from 0.0 to 180.0, though this can be changed by setting MinPosition and MaxPosition.\n"
		"  | For this example, acceleration has been fixed to 1.0Hz, but can be changed in custom code.\n"

		"\nInput a desired position between 0.0 and 180.0 and press ENTER"
		"\nIf your servo doesn't move, but also doesn't cause errors, ensure your RCServo has been enabled.\n"
		"Input Q and press ENTER to quit\n");

	while (!end) {
		if (fgets(buf, 100, stdin) == NULL)
			continue;

		if (buf[0] == 'Q' || buf[0] == 'q')
			end = 1;

		if (buf[0] == '\n' || buf[0] == '\r')
			continue;

		targetPosition = strtod(buf, &endptr);
		if ((*endptr != '\0') && (isspace(*endptr) == 0))
			continue;

		if (targetPosition > 180.0 || targetPosition < 0.0) {
			printf("Position must be between 0 and 180\n");
			continue;
		}

		prc = PhidgetRCServo_setTargetPosition(ch, targetPosition);
		CheckError(prc, "Setting Target Position", (PhidgetHandle *)&ch);
	}

	/*
	* Perform clean up and exit
	*/

	//clear the TargetPositionReached event handler
	prc = PhidgetRCServo_setOnTargetPositionReachedHandler(ch, NULL, NULL);
	CheckError(prc, "Clearing OnTargetPositionReachedHandler", (PhidgetHandle *)&ch);

	printf("\nDone Sampling...\n");

	printf("Cleaning up...\n");
	prc = Phidget_close((PhidgetHandle)ch);
	CheckError(prc, "Closing Channel", (PhidgetHandle *)&ch);
	prc = PhidgetRCServo_delete(&ch);
	CheckError(prc, "Deleting Channel", (PhidgetHandle *)&ch);
	printf("\nExiting...\n");
	printf("Press ENTER to end program.\n");
	getchar();

	return 0;

}
